epsilon = 0.01; /* avoid null width surfaces */
$fn = 100;

//may be overwritten by CLI
box_only=0; /* export box footprint and a ruler to test format */
pcb_only=0; /* export 2D PCB dxf to be imported as edge cuts with Kicad */
cover_only = 0; /* export stl for 3D printing */
glass_only = 0; /* export stl for 3D printing */
/* Utils */

module SquareSectionTore(external_d, section, h)
{
    difference()
    {
        cylinder(r = external_d, h=h);
        translate([0, 0, -epsilon]) cylinder(r = external_d - section, h=h + 2*epsilon);
    }
}

/* complementary parts */

module Box()
{
    r = 67;
    o = 30; /* offset */

    intersection()
    {
        translate([-o,  0, 0]) circle(r = r);
        translate([ o,  0, 0]) circle(r = r);
        translate([ 0, -o, 0]) circle(r = r);
        translate([ 0,  o, 0]) circle(r = r);
    }
}

module Screw()
{
    screw_d = 3.5;
    screw_h = 20;
    screw_head_d = 6.8;
    cylinder(d = screw_d, h=screw_h);
    difference()
    {
        sphere(d = screw_head_d, center = true);
        cylinder(d = screw_head_d + epsilon, h = screw_head_d + epsilon);
        translate([0, 0, -screw_head_d  - epsilon]) cylinder(d = screw_head_d, h = screw_head_d  - screw_head_h - epsilon);
    }
}

module Screws(hole_size)
{
    for (a =[-1:2:1]) translate([a * hole_size/2, 0]) Screw();
}

module PcbScrewPassage(d, w)
{
    difference()
    {
        circle(d = d + w, center = true);
        circle(d = d - w, center = true);
        square(d+w);
        translate([-d - w, -d - w, 0])square(d+w);
    }
}

module Pcb2D(hole_size, screw_passage=true)
{
    $fn=100;
    tongue = 16;
    belly = 50;
    screw_d = 3.5;
    difference()
    {
        hull()
        {
            for (a =[-1:2:1]) translate([a * hole_size/2,0, 0])
            {
                circle(d = tongue, center = true);
            }
            circle(d = belly, center = true);
        };
        for (a =[-1:2:1]) translate([a * hole_size/2, 0, 0]) circle(d = screw_d);
        if(screw_passage)
        {
            PcbScrewPassage(hole_size, screw_d + 0.2);
        }
    }
}

module Pcb(hole_size, pcb_height, screw_passage=true)
{
    linear_extrude(height=pcb_height) 
    {
        Pcb2D(hole_size, screw_passage);
    }
}

/* cover */

module Airflow(thickness, height, radius)
{
    l = 15;
    w = 2;
    angle = 50;
    rotate([0, 0, -angle / 2- 10])
    translate([0, 0, -epsilon])
    linear_extrude(height=height + 2* epsilon, convexity=100)
    { 
        polygon(points=[
            [0,0],
            [l, 0],
            [l, w],
            [w/tan(angle/2), w],
            [cos(angle) * l + sin(angle) * w, sin(angle) * l - cos(angle) * w],
            [cos(angle) * l, sin(angle) * l],
        ]);
    }
}

module CoverBody(radius, height, thickness)
{
    difference()
    {
        SquareSectionTore(radius, thickness, height);
        for (a =[0:15:360]) rotate([0, 0, a]) translate([0, radius - thickness / 2, 0]) Airflow(thickness, height, radius);
        translate([0, 0, height - 2 + epsilon])
        {
            linear_extrude(height=2) 
            {
                Box();
            }
        }
    }
}

module CoverGlass(radius, height, lips_h, lips_w)
{
    $fn=200;
    lens_w = 2;
    lens_h = 1;
    x = 0.3;
    union()
    {
        translate([0, 0, lips_h - epsilon]) cylinder(r = radius, h=height - lips_h + epsilon);
        for (a =[0:lens_w:radius - lips_w - lens_w])
        {
            rotate_extrude() polygon(points=[[a,lips_h],[a+lens_w,lips_h],[a+lens_w,0]]);
        }
    }
}

module CoverPlate(radius, glass_radius, height, lips_h, lips_w)
{
    border_w = 2;
    border_h = 1;
    difference()
    {
        union()
        {
            cylinder(r = radius, h=height);
            cylinder(r = glass_radius + border_w, h=height + border_h);
        }
        translate([0, 0, lips_h]) cylinder(r = glass_radius, h=height + border_h);
        translate([0, 0, -epsilon]) cylinder(r = glass_radius - lips_w, h=height + 2*epsilon);
    }
}

/* assembly */

hole_size = 60; /* normalized box diameter */
glass_radius = 20.2; /* LED light glass radius */
glass_h = 2;
pcb_height = 1.5; /* PCB thickness (NOTE: aluminum PCB) */
screw_head_h = 2.5;
magnet_height=1;
magnet_radius = 3.3;

airflow_height = 2;
plate_height = 2;
cover_height = plate_height + magnet_height + screw_head_h + pcb_height + airflow_height;
plate_glass_lips_h = 1;
plate_glass_lips_w = 2;
plate_glass_border_w = 2;


cover_radius = 45; /* larger cover radius */
cover_thickness = 6; /* cover lateral thickness */


module Cover()
{
    magnet_z = cover_height - pcb_height - screw_head_h - magnet_height; /* 1: margin */
    union()
    {
        CoverPlate(cover_radius, glass_radius, plate_height, plate_glass_lips_h, plate_glass_lips_w);
        translate([0, 0, plate_height]) CoverBody(cover_radius, cover_height - plate_height, cover_thickness);
        for (a =[-1:2:1]) 
        translate([a * hole_size/2, 0, 0]) 
        {
            difference()
            {
                cylinder(r = magnet_radius, h=magnet_z);
                translate([0, 0, magnet_z * 0.9 ]) 
                {
                    SquareSectionTore (magnet_radius * 0.6, magnet_radius * 0.2, magnet_height);
                }
            }
        }
    }
}

module Glass()
{
    color([1, 1, 1, 0.2]) CoverGlass(glass_radius, plate_height, plate_glass_lips_h, plate_glass_lips_w);
}

/* result */

if(box_only)
{
    Box();
     translate([0, 50, 0]) square([200, 10], center = true);
}
else if(pcb_only)
{
    Pcb2D(hole_size);
}
else if(cover_only)
{
    Cover();
}
else if(glass_only)
{
    translate([0, 0, plate_height]) rotate([0, 180, 0]) Glass();
}
else
{
//     Pcb2D(hole_size);
    Cover();
    Glass();
}
